local _,rematch = ...
local frame = RematchFrame
local incFrame = RematchIncomingFrame

-- converts a team into a string and assigns it to rematch.shareString
-- used for export and send
function rematch:GenerateShareString(team)
	if team:len()>128 then
		team = strsub(1,128)
	end
	local info = rematch.info
	wipe(info)
	tinsert(info,team)
	tinsert(info,rematch.savedTeams[team][4] or 0)
	for i=1,3 do
		local speciesID
		if type(rematch.savedTeams[team][i][1])=="string" then
			speciesID = C_PetJournal.GetPetInfoByPetID(rematch.savedTeams[team][i][1])
		else
			speciesID = rematch.savedTeams[team][i][1]
		end
		tinsert(info,speciesID)
		for j=2,4 do
			tinsert(info,rematch.savedTeams[team][i][j])
		end
	end
	rematch.shareString = format("%s:%d:%d:%d:%d:%d:%d:%d:%d:%d:%d:%d:%d:%d:",unpack(info))
end

-- parses a shareString (import string or received via CHAT_MSG_ADDON)
-- if no teamTable is given, returns the name of the team within the string, or nil if a bad string
-- otherwise converts the string into a team-formatted table ["Name"]={{p1,a1,a2,a3},{p2,a1,a2,a3},{p3,a1,a2,a3},npc}
-- NOTE: that teamTable is the PARENT of the key being created/overwritten, not the team itself
-- since the key is generated from the string.
-- returns the name of the team/key if string loaded ok
function rematch:ParseShareString(shareString,teamTable)

	local name,npcID,theRest = shareString:match("^(.-):(%d+):(%d+:%d+:%d+:%d+:%d+:%d+:%d+:%d+:%d+:%d+:%d+:%d+:)")

	if name and teamTable then -- skipping this if not a good string or only wanted name

		if teamTable[name] then -- team already existed, wipe it
			for i=1,3 do
				wipe(teamTable[name][i])
			end
			teamTable[name][4] = nil
		else -- new team, create new table entry for this name
			teamTable[name] = {{},{},{}}
		end

		local s=teamTable[name]
		s[1][1],s[1][2],s[1][3],s[1][4],s[2][1],s[2][2],s[2][3],s[2][4],s[3][1],s[3][2],s[3][3],s[3][4] = strsplit(":",theRest)
		for i=1,3 do
			for j=1,4 do
				s[i][j] = tonumber(s[i][j])
			end
		end
		if npcID~="0" then -- add npc id if it exists
			tinsert(s,tonumber(npcID))
		end

	end

	return name
end

function rematch:ExportTeam()
	rematch.shareMode = 1 -- export
	rematch:GenerateShareString(rematch.selectedTeam)
	rematch:DetailBoxOnChange() -- set and highlight export string
	frame.share:Show()
end

function rematch:ImportTeam()
	rematch.shareMode=2
	frame.share:Show()
	rematch:ClearSelected() -- we'll fill the saved/selected team with what we may import
end

function rematch:DetailBoxOnChange()
	if rematch.shareMode==1 then
		-- in export mode we reset and re-highlight the string on any changes
		local export = frame.share.detailBox.text
		export:SetText(rematch.shareString)
		export:HighlightText()
		export:SetCursorPosition(0)
		return
	end
	-- if this is an import or send, clear any warnings when box emptied
	local detail = frame.share.detailBox.text:GetText()
	detail = (detail or ""):trim()
	if detail:len()==0 then
		rematch:SetShareStatus()
		frame.share.saveButton:Disable()
		if rematch.shareMode==2 and rematch.selectedTeam then
			rematch:ClearSelected() -- also clear any team generated, if any
		end
		return
	end
	if rematch.shareMode==2 then
		-- in import mode we verify that this text is in an import-able format
		-- if so, generate a temporary team and select it
		local name = rematch:ParseShareString(detail)
		if not name then -- verify this is a valid team format
			rematch:SetShareStatus("Sorry, this is not a valid team.")
			frame.share.saveButton:Disable()
			if rematch.selectedTeam then
				rematch:ClearSelected()
			end
			return
		elseif rematch.savedTeams[name] then
			rematch:SetShareStatus("A team already has this name.")
		else
			rematch:SetShareStatus()
		end
		frame.share.saveButton:Enable()
		-- create a temporary team and display as saved pets
		-- this creates a little garbage if they manually type stuff in, but not going to worry; this is for pasting
		local tempTeamName = "~temp~"..detail -- prefixing import string with "~temp~" to make a temporary team
		rematch.selectedTeam = rematch:ParseShareString(tempTeamName,rematch.savedTeams) -- populate temp team from string
		rematch:UpdateSavedPets(1)

	elseif rematch.shareMode==3 then -- send mode
		-- if a status is up, clear it when any changes made to entered name
		if frame.share.statusIcon:IsVisible() then
			rematch:SetShareStatus()
		end
		frame.share.saveButton:Enable() -- if we're here, detail string length is > 0
	end
end

-- displays a message and an icon in the lowerleft corner of frame.share and stops waiting for stuff
-- if no message is given, the status is hidden
-- if no icon is given, then the warning icon is used
function rematch:SetShareStatus(message,icon)
	frame.share.statusIcon:SetTexture(icon and icon or "Interface\\DialogFrame\\UI-Dialog-Icon-AlertNew")
	frame.share.statusText:SetText(message or "")
	frame.share.statusIcon:SetShown(message and true or false)
	frame.share.statusText:SetShown(message and true or false)
	frame:UnregisterEvent("CHAT_MSG_SYSTEM")
	rematch:StopTimer("SendTimeout")
end

-- when enter pressed while share.detailBox.text has focus, or save/send button hit
function rematch:SaveShare()
	if rematch.shareMode==1 then -- exporting
		frame.share:Hide()
	elseif rematch.shareMode==2 then -- importing
		if frame.share.saveButton:IsEnabled() then
			local actualTeam = rematch.selectedTeam:gsub("~temp~","")
			rematch:SaveTeam(actualTeam,rematch.savedTeams[rematch.selectedTeam])
			frame.share:Hide()
		end
	elseif rematch.shareMode==3 then -- sending
		rematch:GenerateShareString(rematch.selectedTeam)
		if rematch.shareString then
			rematch:SetShareStatus("Sending...","Interface\\Icons\\SOR-mail")
			rematch.sendee = frame.share.detailBox.text:GetText():trim()
			frame:RegisterEvent("CHAT_MSG_SYSTEM")
			rematch:StartTimer("SendTimeout",3,rematch.SendTimeout)
			SendAddonMessage("Rematch",rematch.shareString,"WHISPER",rematch.sendee)
		end
	end
end

-- this is registered when we send a team, and unregistered on next SetShareStatus (including panel closing)
function rematch:CHAT_MSG_SYSTEM(message)
	if message==format(ERR_CHAT_PLAYER_NOT_FOUND_S,rematch.sendee) then
		rematch:SetShareStatus("They do not appear to be online.")
	end
end

-- this is run on a 3-second delay after a send, unless a status update shuts it down before then
function rematch:SendTimeout()
	rematch:SetShareStatus("No response. Due to lag or no Rematch?")
end

-- SendAddonMessage from other users to us will trigger this event
function rematch:CHAT_MSG_ADDON(prefix,message,_,sender)
	if prefix~="Rematch" then
		return
	end
	if message=="ok" then -- sendee received team and sent back an ok
		rematch:SetShareStatus("The team was received!","Interface\\RaidFrame\\ReadyCheck-Ready")
	elseif message=="busy" then -- sendee received team but is already looking at a team
		rematch:SetShareStatus("They have another team waiting. Try again later.")
	elseif message=="combat" then -- sendee received team, but they're in combat
		rematch:SetShareStatus("They're in combat. Try again later.")
	elseif message=="block" then -- sendee received team, but has settings.DisableShare checked
		rematch:SetShareStatus("They have team sharing disabled.")
	else -- it's (probably) an incoming team

		-- if our DisableShare is set, tell the sender and leave
		if rematch.settings.DisableShare then
			SendAddonMessage("Rematch","block","WHISPER",sender)
			return
		end
		-- if user is already looking at a team, tell the sender and leave
		if incFrame:IsVisible() then
			SendAddonMessage("Rematch","busy","WHISPER",sender)
			return
		end
		-- if user is in combat (not a game limitation, but it'd be very annoying to get incoming pets in a boss fight)
		if InCombatLockdown() then
			SendAddonMessage("Rematch","combat","WHISPER",sender)
			return
		end
		-- now display the team

		incFrame.greeting:SetText("\124cffffd200"..sender.."\124cffffffff has sent you a battle pet team!")

		if not incFrame.pets then -- if we haven't received a pet this session, set up some stuff
			incFrame.header.text:SetText("Incoming Rematch Team")
			incFrame.warning:SetText("A team with this name already exists.\nEdit the name or hit Save to overwrite it.")
			incFrame.pets = {}
			rematch:CreateConfirmSlots(incFrame.pets,incFrame,16,-70)
		end

		-- parse incoming team into rematch.incomingTeam
		wipe(rematch.incomingTeam)
		rematch.incomingName = rematch:ParseShareString(message,rematch.incomingTeam)

		if not rematch.incomingName then
			return -- we weren't sent a valid team, don't tell sender. they could be using a future version
		end

		-- if we made it this far, team was successfully received. send back "ok"
		SendAddonMessage("Rematch","ok","WHISPER",sender)

		incFrame.incomingAs:SetText(rematch.incomingName) -- put name in editbox
		incFrame.incomingAs:SetCursorPosition(0)

		rematch:WipePetFrames(incFrame.pets)

		-- fill slots with pets from generated team
		local team = rematch.incomingTeam[rematch.incomingName]
		for i=1,3 do
			local speciesName,icon = C_PetJournal.GetPetInfoBySpeciesID(team[i][1])
			if speciesName then -- all incoming petIDs are speciesIDs
				local _,foundPetID = C_PetJournal.FindPetIDByName(speciesName)
				if foundPetID then
					team[i][1] = foundPetID
				end
				incFrame.pets[i].petID = team[i][1]
				incFrame.pets[i].icon:SetTexture(icon)
				incFrame.pets[i].icon:Show()
				rematch:DesaturateIfMissing(incFrame.pets[i].icon,not foundPetID)
				for j=1,3 do
					incFrame.pets[i].ability[j].abilityID = team[i][j+1]
					incFrame.pets[i].ability[j].icon:SetTexture((select(3,C_PetBattles.GetAbilityInfoByID(team[i][j+1]))))
					incFrame.pets[i].ability[j].icon:Show()
					rematch:DesaturateIfMissing(incFrame.pets[i].ability[j].icon,not foundPetID)
				end
			elseif team[i][1]==0 then -- if this is a leveling slot, handle it differently
				incFrame.pets[i].petID = 0
				incFrame.pets[i].icon:SetTexture("Interface\\Icons\\PetJournalPortrait")
				incFrame.pets[i].icon:Show()
				for j=1,3 do
					incFrame.pets[i].ability[j].abilityID = team[i][j+1]
				end
			end
		end
		incFrame:Show()
		rematch.IncomingAsOnTextChanged(incFrame.incomingAs)
	end
end

-- when incoming editbox changes
function rematch:IncomingAsOnTextChanged()
	local name = (self:GetText() or ""):trim()
	incFrame.allClear:Show()
	incFrame.warning:Hide()
	if name:len()==0 then
		incFrame.saveButton:Disable()
		return
	elseif rematch.savedTeams[name] then
		incFrame.allClear:Hide()
		incFrame.warning:Show()
	end
	incFrame.saveButton:Enable()
end

-- final save from incoming dialog; create a saved[] team and update drawer if it's up
function rematch:SaveIncoming()
	if incFrame.saveButton:IsEnabled() then
		-- name the team what's currently in the incomingAs editbox
		local name = incFrame.incomingAs:GetText():trim()
		rematch:SaveTeam(name,rematch.incomingTeam[rematch.incomingName])
		frame.share:Hide()
	end
end
